@@ -30,6 +30,7 @@ class _TeeOut(object):
 
     def write(self, data):
         """Writes data to the original stdout and the buffer."""
+        data = data.replace('\001', '').replace('\002', '')
         self.stdout.write(data)
         self.buffer.write(data)
 
@@ -62,6 +63,7 @@ class _TeeErr(object):
 
     def write(self, data):
         """Writes data to the original stderr and the buffer."""
+        data = data.replace('\001', '').replace('\002', '')
         self.stderr.write(data)
         self.buffer.write(data)
 
@@ -117,6 +119,11 @@ class BaseShell(object):
         self.need_more_lines = False
         self.default('')
 
+    def singleline(self, **kwargs):
+        """Reads a single line of input from the shell."""
+        msg = '{0} has not implemented singleline().'
+        raise RuntimeError(msg.format(self.__class__.__name__))
+
     def precmd(self, line):
         """Called just before execution of line."""
         return line if self.need_more_lines else line.lstrip()
