@@ -7,6 +7,7 @@ special Python builtins module.
 import os
 import re
 import sys
+import stat
 import time
 import shlex
 import atexit
@@ -281,6 +282,49 @@ def iglobpath(s, ignore_case=False):
 RE_SHEBANG = re.compile(r'#![ \t]*(.+?)$')
 
 
+def _is_executable_file(path):
+    """Checks that path is an executable regular file (or a symlink to a file).
+
+    This is roughly ``os.path isfile(path) and os.access(path, os.X_OK)``, but
+    on some platforms :func:`os.access` gives us the wrong answer, so this
+    checks permission bits directly.
+    """
+    # follow symlinks,
+    fpath = os.path.realpath(path)
+
+    # return False for non-files (directories, fifo, etc.)
+    if not os.path.isfile(fpath):
+        return False
+
+    # On Solaris, etc., "If the process has appropriate privileges, an
+    # implementation may indicate success for X_OK even if none of the
+    # execute file permission bits are set."
+    #
+    # For this reason, it is necessary to explicitly check st_mode
+
+    # get file mode using os.stat, and check if `other',
+    # that is anybody, may read and execute.
+    mode = os.stat(fpath).st_mode
+    if mode & stat.S_IROTH and mode & stat.S_IXOTH:
+        return True
+
+    # get current user's group ids, and check if `group',
+    # when matching ours, may read and execute.
+    user_gids = os.getgroups() + [os.getgid()]
+    if (os.stat(fpath).st_gid in user_gids and
+            mode & stat.S_IRGRP and mode & stat.S_IXGRP):
+        return True
+
+    # finally, if file owner matches our effective userid,
+    # check if `user', may read and execute.
+    user_gids = os.getgroups() + [os.getgid()]
+    if (os.stat(fpath).st_uid == os.geteuid() and
+            mode & stat.S_IRUSR and mode & stat.S_IXUSR):
+        return True
+
+    return False
+
+
 def _get_runnable_name(fname):
     if os.path.isfile(fname) and fname != os.path.basename(fname):
         return fname
@@ -294,7 +338,9 @@ def _get_runnable_name(fname):
                     if fname == froot and ext.upper() in PATHEXT:
                         return os.path.join(d, dirfile)
             if fname in files:
-                return os.path.join(d, fname)
+                path = os.path.join(d, fname)
+                if _is_executable_file(path):
+                    return path
     return None
 
 
@@ -528,11 +574,15 @@ def run_subproc(cmds, captured=True):
             stdout = streams['stdout']
         elif captured or ix != last_cmd:
             stdout = PIPE
+        elif builtins.__xonsh_stdout_uncaptured__ is not None:
+            stdout = builtins.__xonsh_stdout_uncaptured__
         else:
             stdout = None
         # set standard error
         if 'stderr' in streams:
             stderr = streams['stderr']
+        elif builtins.__xonsh_stderr_uncaptured__ is not None:
+            stderr = builtins.__xonsh_stderr_uncaptured__
         uninew = (ix == last_cmd) and (not captured)
         alias = builtins.aliases.get(cmd[0], None)
         if (alias is None
@@ -625,7 +675,7 @@ def run_subproc(cmds, captured=True):
     hist.last_cmd_rtn = prev_proc.returncode
     if write_target is None:
         # get output
-        output = ''
+        output = b''
         if prev_proc.stdout not in (None, sys.stdout):
             output = prev_proc.stdout.read()
         if captured:
@@ -664,13 +714,13 @@ def ensure_list_of_strs(x):
     return rtn
 
 
-def load_builtins(execer=None):
+def load_builtins(execer=None, config=None):
     """Loads the xonsh builtins into the Python builtins. Sets the
     BUILTINS_LOADED variable to True.
     """
     global BUILTINS_LOADED, ENV
     # private built-ins
-    builtins.__xonsh_env__ = ENV = Env(default_env())
+    builtins.__xonsh_env__ = ENV = Env(default_env(config=config))
     builtins.__xonsh_ctx__ = {}
     builtins.__xonsh_help__ = helper
     builtins.__xonsh_superhelp__ = superhelper
@@ -678,6 +728,8 @@ def load_builtins(execer=None):
     builtins.__xonsh_glob__ = globpath
     builtins.__xonsh_expand_path__ = expand_path
     builtins.__xonsh_exit__ = False
+    builtins.__xonsh_stdout_uncaptured__ = None
+    builtins.__xonsh_stderr_uncaptured__ = None
     if hasattr(builtins, 'exit'):
         builtins.__xonsh_pyexit__ = builtins.exit
         del builtins.exit
@@ -729,6 +781,8 @@ def unload_builtins():
              '__xonsh_glob__',
              '__xonsh_expand_path__',
              '__xonsh_exit__',
+             '__xonsh_stdout_uncaptured__',
+             '__xonsh_stderr_uncaptured__',
              '__xonsh_pyexit__',
              '__xonsh_pyquit__',
              '__xonsh_subproc_captured__',
