@@ -16,11 +16,8 @@ from xonsh.built_ins import load_builtins, unload_builtins
 class Execer(object):
     """Executes xonsh code in a context."""
 
-    def __init__(self,
-                 filename='<xonsh-code>',
-                 debug_level=0,
-                 parser_args=None,
-                 unload=True):
+    def __init__(self, filename='<xonsh-code>', debug_level=0, parser_args=None,
+                 unload=True, config=None):
         """Parameters
         ----------
         filename : str, optional
@@ -31,6 +28,8 @@ class Execer(object):
             Arguments to pass down to the parser.
         unload : bool, optional
             Whether or not to unload xonsh builtins upon deletion.
+        config : str, optional
+            Path to configuration file.
         """
         parser_args = parser_args or {}
         self.parser = Parser(**parser_args)
@@ -38,7 +37,7 @@ class Execer(object):
         self.debug_level = debug_level
         self.unload = unload
         self.ctxtransformer = ast.CtxAwareTransformer(self.parser)
-        load_builtins(execer=self)
+        load_builtins(execer=self, config=config)
 
     def __del__(self):
         if self.unload:
@@ -186,6 +185,11 @@ class Execer(object):
                     if prev_indent == curr_indent:
                         raise original_error
                 maxcol = self._find_next_break(line, last_error_col)
+                if line.startswith('$['):
+                    # if we have already wrapped this in subproc tokens
+                    # and it still doesn't work, adding more won't help
+                    # anything
+                    raise original_error
                 sbpline = subproc_toks(line,
                                        returnline=True,
                                        maxcol=maxcol,
